package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dao.UserOrderRefundDao;
import com.yonge.cooleshow.biz.dal.dto.req.AuthOperaReq;
import com.yonge.cooleshow.biz.dal.dto.req.OrderRefundReq;
import com.yonge.cooleshow.biz.dal.dto.search.OrderDetailSearch;
import com.yonge.cooleshow.biz.dal.dto.search.UserOrderRefundSearch;
import com.yonge.cooleshow.biz.dal.entity.UserOrderDetail;
import com.yonge.cooleshow.biz.dal.entity.UserOrderPayment;
import com.yonge.cooleshow.biz.dal.entity.UserOrderRefund;
import com.yonge.cooleshow.biz.dal.entity.UserOrderRefundBill;
import com.yonge.cooleshow.biz.dal.enums.AuthStatusEnum;
import com.yonge.cooleshow.biz.dal.enums.GoodTypeEnum;
import com.yonge.cooleshow.biz.dal.enums.OrderStatusEnum;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.UserOrderDetailVo;
import com.yonge.cooleshow.biz.dal.vo.UserOrderRefundVo;
import com.yonge.cooleshow.biz.dal.vo.UserOrderVo;
import com.yonge.cooleshow.biz.dal.vo.res.RefundCreateRes;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.service.IdGeneratorService;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.result.BaseResult;
import com.yonge.toolset.base.util.StringUtil;
import com.yonge.toolset.payment.base.enums.PaymentClientEnum;
import com.yonge.toolset.payment.base.enums.TradeStatusEnum;
import com.yonge.toolset.payment.base.model.RefundBill;
import com.yonge.toolset.payment.base.model.callback.PaymentCallBack;
import com.yonge.toolset.payment.base.model.callback.RefundPaymentCallBack;
import com.yonge.toolset.payment.core.service.PaymentClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class UserOrderRefundServiceImpl extends ServiceImpl<UserOrderRefundDao, UserOrderRefund> implements UserOrderRefundService {
    private final static Logger log = LoggerFactory.getLogger(UserOrderRefundServiceImpl.class);

    @Autowired
    private UserOrderService orderService;
    @Autowired
    private UserOrderPaymentService orderPaymentService;
    @Autowired
    private UserOrderRefundBillService refundBillService;
    @Autowired
    private UserOrderDetailService orderDetailService;
    @Autowired
    private CourseGroupService courseGroupService;
    @Autowired
    private IdGeneratorService idGeneratorService;
    @Autowired
    private PaymentClient paymentClient;

    //验证是否可以退款,获取退款金额信息
    private static final Map<GoodTypeEnum, Function<OrderRefundReq, HttpResponseResult<RefundCreateRes>>> refundCreate = new HashMap<>();
    //插入退款后执行
    private static final Map<GoodTypeEnum, Consumer<UserOrderDetailVo>> refundAfter = new HashMap<>();
    //退款完成后执行
    private static final Map<GoodTypeEnum, Consumer<UserOrderDetailVo>> refundSuccess = new HashMap<>();
    //退款未通过后执行
    private static final Map<GoodTypeEnum, Consumer<UserOrderDetailVo>> refundCancel = new HashMap<>();
    //退款失败执行
    private static final Map<GoodTypeEnum, Consumer<UserOrderDetailVo>> refundFailed = new HashMap<>();

    @PostConstruct
    private void init() {
        /**********退费单生成前******************/
        //直播课退费
        refundCreate.put(GoodTypeEnum.LIVE, courseGroupService::refundCreate);

        refundSuccess.put(GoodTypeEnum.LIVE, courseGroupService::refundSuccess);
    }

    @Override
    public UserOrderRefundVo detail(Long id) {
        UserOrderRefundVo detail = baseMapper.detail(id, null);
        return detail;
    }

    @Override
    public UserOrderRefundVo detail(Long id, Long userId) {
        UserOrderRefundVo detail = baseMapper.detail(id, userId);
        return detail;
    }

    @Override
    public IPage<UserOrderRefundVo> selectPage(IPage<UserOrderRefundVo> page, UserOrderRefundSearch query) {
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<Boolean> orderRefund(OrderRefundReq refundReq) {
        UserOrderVo detail = orderService.detail(refundReq.getOrderId());
        if (null == detail || CollectionUtils.isEmpty(detail.getOrderDetailList())) {
            return HttpResponseResult.failed("未找到订单信息");
        }
        if (!OrderStatusEnum.PAID.equals(detail.getStatus())) {
            return HttpResponseResult.failed("订单状态异常");
        }
        if (StringUtil.isEmpty(detail.getTransNo()) && StringUtil.isEmpty(detail.getPaymentNo())) {
            return HttpResponseResult.failed("未找到订单付款信息");
        }
        UserOrderPayment orderPayment = orderPaymentService.detailByTransNoOrPaymentNo(detail.getTransNo(), detail.getPaymentNo());
        if (null == orderPayment || !TradeStatusEnum.succeeded.equals(orderPayment.getStatus())) {
            return HttpResponseResult.failed("订单付款状态异常");
        }
        if (CollectionUtils.isEmpty(refundReq.getOredrDetilIds())) {
            //查询订单下未退款的所有详情订单
            List<UserOrderDetail> orderDetails = baseMapper.selectOrderRefundDetils(refundReq.getOrderId());

            refundReq.setOredrDetilIds(orderDetails.stream().map(UserOrderDetail::getId).collect(Collectors.toList()));
        }
        //退款的详情id
        List<Long> detilIds = new ArrayList<>();
        //退款金额
        BigDecimal actualPrice = BigDecimal.ZERO;
        for (UserOrderDetailVo vo : detail.getOrderDetailList()) {
            List<Long> collect = refundReq.getOredrDetilIds().stream().filter(o -> o.equals(vo.getId())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(collect)) {

                Function<OrderRefundReq, HttpResponseResult<RefundCreateRes>> refundCreateFunction = refundCreate.get(vo.getGoodType());
                if (!Objects.isNull(refundCreateFunction)) {
                    refundReq.setOredrDetil(vo);
                    HttpResponseResult<RefundCreateRes> apply = refundCreateFunction.apply(refundReq);
                    if (apply.getStatus()) {
                        detilIds.add(vo.getId());
                        actualPrice = actualPrice.add(apply.getData().getActualPrice());
                    }
                } else {
                    detilIds.add(vo.getId());
                    actualPrice = actualPrice.add(vo.getActualPrice());
                }
            }
        }
        if (BigDecimal.ZERO.compareTo(actualPrice) >= 0) {
            throw new BizException("退款金额不能为0");
        }

        String join = StringUtil.join(detilIds, ",");

        UserOrderRefund orderRefunds = new UserOrderRefund();
        orderRefunds.setUserId(detail.getUserId());
        orderRefunds.setOrderId(detail.getId());
        orderRefunds.setOrderNo(detail.getOrderNo());
        orderRefunds.setOredrDetilIds(join);
        orderRefunds.setStatus(AuthStatusEnum.DOING);
        orderRefunds.setApplyAmount(actualPrice);
        orderRefunds.setReason(refundReq.getReason());
        save(orderRefunds);

        for (UserOrderDetailVo vo : detail.getOrderDetailList()) {
            List<Long> collect = refundReq.getOredrDetilIds().stream().filter(o -> o.equals(vo.getId())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(collect)) {
                Consumer<UserOrderDetailVo> refundAfterConsumer = refundAfter.get(vo.getGoodType());
                if (!Objects.isNull(refundAfterConsumer)) {
                    refundAfterConsumer.accept(vo);
                }
            }
        }
        return HttpResponseResult.succeed(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<UserOrderRefundBill> doAuth(AuthOperaReq authOperaReq, SysUser user) {
        UserOrderRefund orderRefund = baseMapper.selectById(authOperaReq.getId());
        if (!AuthStatusEnum.DOING.equals(orderRefund.getStatus())) {
            return HttpResponseResult.failed("退款单已审核");
        }
        String oredrDetilIds = orderRefund.getOredrDetilIds();
        if (StringUtil.isEmpty(oredrDetilIds)) {
            return HttpResponseResult.succeed();
        }

        orderRefund.setStatus(authOperaReq.getPass() ? AuthStatusEnum.PASS : AuthStatusEnum.UNPASS);
        orderRefund.setActualAmount(orderRefund.getApplyAmount());
        orderRefund.setOperateId(user.getId());
        orderRefund.setOperateReason(authOperaReq.getReason());
        orderRefund.setUpdateTime(new Date());
        updateById(orderRefund);

        if (authOperaReq.getPass()) {
            //通过调用退款接口
            return doOrderRefund(orderRefund);
        } else {
            List<String> detilIds = Arrays.asList(oredrDetilIds.split(","));
            for (String id : detilIds) {
                UserOrderDetailVo detail = orderDetailService.detail(Long.parseLong(id));
                if (null == detail) {
                    continue;
                }
                //调用业务方法
                Consumer<UserOrderDetailVo> refundCancelConsumer = refundCancel.get(detail.getGoodType());
                if (!Objects.isNull(refundCancelConsumer)) {
                    refundCancelConsumer.accept(detail);
                }
            }
        }
        return HttpResponseResult.succeed();
    }


    @Transactional
    public HttpResponseResult<UserOrderRefundBill> orderRefund(String orderNo, String reason) {
        UserOrderVo detail = orderService.detail(orderNo, null);

        if (null == detail || CollectionUtils.isEmpty(detail.getOrderDetailList())) {
            return HttpResponseResult.failed("未找到订单信息");
        }
        if (!OrderStatusEnum.PAID.equals(detail.getStatus())) {
            return HttpResponseResult.failed("订单状态异常");
        }

        UserOrderPayment orderPayment = orderPaymentService.detailByTransNoOrPaymentNo(
                detail.getTransNo(), detail.getPaymentNo());
        if (null == orderPayment || !TradeStatusEnum.succeeded.equals(orderPayment.getStatus())) {
            return HttpResponseResult.failed("订单付款状态异常");
        }
        //查询订单下未退款的所有详情订单
        List<UserOrderDetail> orderDetails = baseMapper.selectOrderRefundDetils(detail.getId());

        //退款的详情id
        List<Long> detilIds = orderDetails.stream().map(UserOrderDetail::getId).collect(Collectors.toList());

        OrderRefundReq orderRefundReq = new OrderRefundReq();
        orderRefundReq.setOrderId(detail.getId());
        orderRefundReq.setUserId(detail.getUserId());
        orderRefundReq.setReason(reason);
        orderRefundReq.setOredrDetilIds(detilIds);
        //退款金额
        BigDecimal actualPrice = BigDecimal.ZERO;
        for (UserOrderDetailVo vo : detail.getOrderDetailList()) {
            Optional<Long> first = detilIds.stream().filter(o -> o.equals(vo.getId())).findFirst();
            if (!first.isPresent()) {
                continue;
            }

            Function<OrderRefundReq, HttpResponseResult<RefundCreateRes>> refundCreateFunction = refundCreate.get(vo.getGoodType());
            if (!Objects.isNull(refundCreateFunction)) {
                orderRefundReq.setOredrDetil(vo);
                HttpResponseResult<RefundCreateRes> apply = refundCreateFunction.apply(orderRefundReq);
                if (apply.getStatus()) {
                    detilIds.add(vo.getId());
                    actualPrice = actualPrice.add(apply.getData().getActualPrice());
                }
            } else {
                detilIds.add(vo.getId());
                actualPrice = actualPrice.add(vo.getActualPrice());
            }
        }
        String join = StringUtil.join(detilIds, ",");

        UserOrderRefund orderRefunds = new UserOrderRefund();
        orderRefunds.setUserId(detail.getUserId());
        orderRefunds.setOrderId(detail.getId());
        orderRefunds.setOrderNo(detail.getOrderNo());
        orderRefunds.setOredrDetilIds(join);
        orderRefunds.setStatus(AuthStatusEnum.PASS);
        orderRefunds.setApplyAmount(actualPrice);
        orderRefunds.setReason(reason);
        save(orderRefunds);

        for (UserOrderDetailVo vo : detail.getOrderDetailList()) {
            List<Long> collect = orderRefundReq.getOredrDetilIds().stream().filter(o -> o.equals(vo.getId())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(collect)) {
                Consumer<UserOrderDetailVo> refundAfterConsumer = refundAfter.get(vo.getGoodType());
                if (!Objects.isNull(refundAfterConsumer)) {
                    refundAfterConsumer.accept(vo);
                }
            }
        }

        return doOrderRefund(orderRefunds);
    }


    @Override
    public void refundPaymentCallBack(PaymentCallBack data) {
        if (TradeStatusEnum.succeeded.equals(data.getStatus())) {//退款成功
            paymentRefundSucceededHandle(data);
        } else if (TradeStatusEnum.failed.equals(data.getStatus())) {//退款失败
            paymentRefundFailedHandle(data, data.getErrMsg());
        }
    }

    private HttpResponseResult<UserOrderRefundBill> doOrderRefund(UserOrderRefund orderRefund) {
        UserOrderPayment payment = orderPaymentService.getOne(Wrappers.<UserOrderPayment>lambdaQuery()
                .eq(UserOrderPayment::getOrderNo, orderRefund.getOrderNo())
                .eq(UserOrderPayment::getStatus, TradeStatusEnum.succeeded)
        );

        if (null == payment || !TradeStatusEnum.succeeded.equals(payment.getStatus())) {
            throw new BizException("订单状态异常");
        }

        //单号生成
        Long billNo = idGeneratorService.generatorId("billNo");

        RefundBill refundBill = new RefundBill(payment.getOpenType(),
                PaymentClientEnum.valueOf(payment.getPaymentClient()), payment.getPayChannel());
        refundBill.setTradeNo(payment.getTransNo());
        refundBill.setPaymentNo(payment.getPaymentNo());
        refundBill.setRefundNo(billNo.toString());
        refundBill.setOrderAmt(payment.getPayAmt());
        refundBill.setRefundAmt(orderRefund.getActualAmount());
        refundBill.setReason(orderRefund.getReason());

        BaseResult<RefundBill> refundBillBaseResult = paymentClient.refundPayment(refundBill);
        log.info("退款返回: {} ", JSONObject.toJSONString(refundBillBaseResult));
        //入退款单表
        UserOrderRefundBill orderRefundBill = new UserOrderRefundBill();
        orderRefundBill.setRefundId(orderRefund.getId());
        orderRefundBill.setBillNo(billNo.toString());
        orderRefundBill.setRefundAmt(orderRefund.getActualAmount());
        if (refundBillBaseResult.getStatus()) {
            orderRefundBill.setTransNo(refundBillBaseResult.getData().getId());
            orderRefundBill.setStatus(TradeStatusEnum.pending);
        } else {
            orderRefundBill.setStatus(TradeStatusEnum.failed);
            orderRefundBill.setPayFailMsg(refundBillBaseResult.getMsg());
        }
        refundBillService.save(orderRefundBill);
        return HttpResponseResult.succeed(orderRefundBill);
    }

    /***
     * 处理回调-退款成功
     * @author liweifan
     * @param: data
     * @updateTime 2022/4/27 15:09
     */
    private void paymentRefundSucceededHandle(PaymentCallBack data) {
        RefundPaymentCallBack refundPaymentCallBack = data.getRefundPaymentCallBack();

        UserOrderRefundBill one = refundBillService.getOne(Wrappers.<UserOrderRefundBill>lambdaQuery().and(
                wrapper -> wrapper.eq(UserOrderRefundBill::getTransNo, refundPaymentCallBack.getTransNo())
                        .or().eq(UserOrderRefundBill::getBillNo, refundPaymentCallBack.getBillNo())
        ));
        //未找到退款单的和已经退款的不能再进行退款
        if (null == one || !TradeStatusEnum.pending.equals(one.getStatus())) {
            return;
        }
        one.setTransNo(refundPaymentCallBack.getTransNo());
        one.setBackRefundAmt(new BigDecimal(refundPaymentCallBack.getRefundAmt()));
        one.setStatus(TradeStatusEnum.succeeded);
        one.setUpdateTime(new Date());
        refundBillService.updateById(one);

        //处理业务
        UserOrderRefund orderRefund = getById(one.getRefundId());
        if (StringUtil.isEmpty(orderRefund.getOredrDetilIds())) {
            List<Long> ids = new ArrayList<>();
            for (String id : orderRefund.getOredrDetilIds().split(",")) {
                ids.add(Long.parseLong(id));
            }
            OrderDetailSearch search = new OrderDetailSearch();
            search.setIds(ids);
            search.setOrderNo(orderRefund.getOrderNo());
            List<UserOrderDetailVo> userOrderDetails = orderDetailService.selectList(search);

            for (UserOrderDetailVo vo : userOrderDetails) {
                Consumer<UserOrderDetailVo> refundAfterConsumer = refundSuccess.get(vo.getGoodType());
                if (!Objects.isNull(refundAfterConsumer)) {
                    refundAfterConsumer.accept(vo);
                }
            }

        }
    }

    /***
     * 处理回调-退款失败
     * @author liweifan
     * @param: data
     * @updateTime 2022/4/27 15:09
     */
    private void paymentRefundFailedHandle(PaymentCallBack data, String errMsg) {
        RefundPaymentCallBack refundPaymentCallBack = data.getRefundPaymentCallBack();

        UserOrderRefundBill one = refundBillService.getOne(Wrappers.<UserOrderRefundBill>lambdaQuery().and(
                wrapper -> wrapper.eq(UserOrderRefundBill::getTransNo, refundPaymentCallBack.getTransNo())
                        .or().eq(UserOrderRefundBill::getBillNo, refundPaymentCallBack.getBillNo())
        ));

        //未找到退款单的和已经退款的不能再进行退款
        if (null == one || !TradeStatusEnum.pending.equals(one.getStatus())) {
            return;
        }
        one.setTransNo(refundPaymentCallBack.getTransNo());
        one.setBackRefundAmt(new BigDecimal(refundPaymentCallBack.getRefundAmt()));
        one.setStatus(TradeStatusEnum.failed);
        one.setPayFailMsg(errMsg);
        one.setUpdateTime(new Date());
        refundBillService.updateById(one);

        //处理业务
        UserOrderRefund orderRefund = getById(one.getRefundId());
        if (StringUtil.isEmpty(orderRefund.getOredrDetilIds())) {
            List<Long> ids = new ArrayList<>();
            for (String id : orderRefund.getOredrDetilIds().split(",")) {
                ids.add(Long.parseLong(id));
            }
            OrderDetailSearch search = new OrderDetailSearch();
            search.setIds(ids);
            search.setOrderNo(orderRefund.getOrderNo());
            List<UserOrderDetailVo> userOrderDetails = orderDetailService.selectList(search);

            for (UserOrderDetailVo vo : userOrderDetails) {
                Consumer<UserOrderDetailVo> refundAfterConsumer = refundFailed.get(vo.getGoodType());
                if (!Objects.isNull(refundAfterConsumer)) {
                    refundAfterConsumer.accept(vo);
                }
            }

        }
    }
}