package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.MessageTypeEnum;
import com.yonge.cooleshow.biz.dal.service.MemberPriceSettingsService;
import com.yonge.cooleshow.biz.dal.service.SysMessageService;
import com.yonge.toolset.thirdparty.message.MessageSenderPluginContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yonge.cooleshow.biz.dal.entity.VipCardRecord;
import com.yonge.cooleshow.biz.dal.vo.VipCardRecordVo;
import com.yonge.cooleshow.biz.dal.dto.search.VipCardRecordSearch;
import com.yonge.cooleshow.biz.dal.dao.VipCardRecordDao;
import com.yonge.cooleshow.biz.dal.service.VipCardRecordService;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Service
public class VipCardRecordServiceImpl extends ServiceImpl<VipCardRecordDao, VipCardRecord> implements VipCardRecordService {
    private final static Logger log = LoggerFactory.getLogger(VipCardRecordServiceImpl.class);

    @Autowired
    private SysMessageService sysMessageService;

    @Override
    public VipCardRecordVo detail(Long orderDetilId) {
        return baseMapper.detail(orderDetilId, null);
    }

    @Override
    public VipCardRecordVo detail(Long orderDetilId, Long userId) {
        return baseMapper.detail(orderDetilId, userId);
    }

    @Override
    public IPage<VipCardRecordVo> selectPage(IPage<VipCardRecordVo> page, VipCardRecordSearch query) {
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    @Override
    public void pollExpireMsg() {
        //提前三天提示
        List<VipCardRecordVo> temporaryRecords = baseMapper.selectTemporaryRecord();
        //标识
        Map<Long, Long> temporaryFlagMap = new HashMap<>();
        for (VipCardRecordVo record : temporaryRecords) {
            if (null != temporaryFlagMap.get(record.getUserId())) {
                continue;
            }
            temporaryFlagMap.put(record.getUserId(), record.getUserId());
            //发送消息
            temporary3DaysSend(record.getUserId(), record.getPhone());
            record.setMsgStatus(1);
            record.setUpdateTime(new Date());

            VipCardRecord vipCardRecord = new VipCardRecord();
            BeanUtils.copyProperties(record, vipCardRecord);
            baseMapper.updateById(vipCardRecord);
        }

        //到期提示
        List<VipCardRecordVo> expireRecords = baseMapper.selectExpireRecord();
        //标识
        Map<Long, Long> expireFlagMap = new HashMap<>();
        for (VipCardRecordVo record : expireRecords) {
            if (null != expireFlagMap.get(record.getUserId())) {
                continue;
            }
            expireFlagMap.put(record.getUserId(), record.getUserId());
            //发送消息
            expireSend(record.getUserId(), record.getPhone());
            record.setMsgStatus(2);
            record.setUpdateTime(new Date());

            VipCardRecord vipCardRecord = new VipCardRecord();
            BeanUtils.copyProperties(record, vipCardRecord);
            baseMapper.updateById(vipCardRecord);
        }

    }


    // 发送会员到期3天消息推送
    private void temporary3DaysSend(Long userId, String phone) {
        Map<Long, String> receivers = new HashMap<>();
        receivers.put(userId, phone);
        try {
            String url = sysMessageService.selectConfigUrl(MessageTypeEnum.VIP_EXPIRE_THIRTY_DAY.getCode());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.VIP_EXPIRE_THIRTY_DAY,
                    receivers, null, 0, url, ClientEnum.STUDENT.getCode());
        } catch (Exception e) {
            log.error("会员到期3天极光消息推送异常,userId={}", userId);
        }

        try {
            String url = sysMessageService.selectConfigUrl(MessageTypeEnum.SMS_VIP_EXPIRE_THIRTY_DAY.getCode());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.SMS_VIP_EXPIRE_THIRTY_DAY,
                    receivers, null, 0, null, ClientEnum.STUDENT.getCode(), url);
        } catch (Exception e) {
            log.error("会员到期3天短信消息推送异常,userId={}", userId);
        }
    }

    // 发送会员到期消息推送
    private void expireSend(Long userId, String phone) {
        Map<Long, String> receivers = new HashMap<>();
        receivers.put(userId, phone);
        try {
            String url = sysMessageService.selectConfigUrl(MessageTypeEnum.VIP_EXPIRE.getCode());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.VIP_EXPIRE,
                    receivers, null, 0, url, ClientEnum.STUDENT.getCode());
        } catch (Exception e) {
            log.error("会员到期3天极光消息推送异常,userId={}", userId);
        }

        try {
            String url = sysMessageService.selectConfigUrl(MessageTypeEnum.SMS_VIP_EXPIRE.getCode());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.SMS_VIP_EXPIRE,
                    receivers, null, 0, null, ClientEnum.STUDENT.getCode(), url);
        } catch (Exception e) {
            log.error("会员到期3天短信消息推送异常,userId={}", userId);
        }
    }
}