package com.yonge.cooleshow.teacher.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.common.enums.SysUserType;
import com.yonge.cooleshow.biz.dal.entity.ContractTemplate;
import com.yonge.cooleshow.biz.dal.entity.SysUserContractRecord;
import com.yonge.cooleshow.biz.dal.enums.ContractTypeEnum;
import com.yonge.cooleshow.biz.dal.enums.DealStatusEnum;
import com.yonge.cooleshow.biz.dal.sdk.WithdrawSdk;
import com.yonge.cooleshow.biz.dal.service.ContractTemplateService;
import com.yonge.cooleshow.biz.dal.service.SysUserContractRecordService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.service.IdGeneratorService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * 用户协议记录表-一个用户一种协议一个版本一条记录(SysUserContractRecord)表控制层
 *
 * @author hgw
 * @since 2022-05-07 15:04:05
 */
@Api(tags = "用户协议记录表-一个用户一种协议一个版本一条记录")
@RestController
@RequestMapping("/sysUserContractRecord")
public class SysUserContractRecordController extends BaseController {
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private ContractTemplateService contractTemplateService;
    @Resource
    private SysUserContractRecordService sysUserContractRecordService;
    @Autowired
    private WithdrawSdk withdrawSdk;
    @Autowired
    private IdGeneratorService idGeneratorService;

    @ApiOperation("判断用户是否签署协议")
    @GetMapping(value = "/checkContractSign", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public HttpResponseResult<Boolean> checkContractSign(@ApiParam(value = "业务类型 REGISTER 注册 COURSES 课程购买 PRODUCT 产品 WITHDRAW 提现", required = true) @RequestParam("contractType") ContractTypeEnum contractType) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return sysUserContractRecordService.checkContractSign(user.getId(), contractType);
    }

    @ApiOperation("查询最新启用的协议模板")
    @GetMapping(value = "/queryLatestContractTemplate")
    public HttpResponseResult<ContractTemplate> queryLatestContractTemplate(@ApiParam(value = "业务类型 REGISTER 注册 COURSES 课程购买 PRODUCT 产品 WITHDRAW 提现", required = true) @RequestParam("contractType") ContractTypeEnum contractType) {
        return succeed(contractTemplateService.queryLatestContractTemplate(contractType));
    }

    @ApiOperation("签署协议")
    @GetMapping(value = "/sign", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public HttpResponseResult<Boolean> sign(
            @ApiParam(value = "模板id", required = true) @RequestParam("templateId") Integer templateId) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        //是否签署标识，签署了才入库
        boolean flag = false;

        //查询库中正在签署的协议
        SysUserContractRecord contractRecord = sysUserContractRecordService.getOne(Wrappers.<SysUserContractRecord>lambdaQuery()
                .eq(SysUserContractRecord::getUserId, user.getId())
                .eq(SysUserContractRecord::getContractId, templateId)
                .in(SysUserContractRecord::getStatus, DealStatusEnum.SCCESS.getCode(), DealStatusEnum.DOING.getCode()));

        //未签署协议
        if (null == contractRecord) {
            contractRecord = new SysUserContractRecord();
            Long contractNo = idGeneratorService.generatorId("contractNo");
            contractRecord.setContractNo(contractNo.toString());
            contractRecord.setUserId(user.getId());

            ContractTemplate contractTemplate = contractTemplateService.get(templateId);
            if(null == contractTemplate){
                return failed(HttpStatus.FORBIDDEN, "未找到协议");
            }
            contractRecord.setContractId(templateId);

            if (ContractTypeEnum.WITHDRAW.getCode().equals(contractTemplate.getType())) {
                //签署提现协议
                contractRecord.setUserType(SysUserType.TEACHER.getCode());
                contractRecord.setStatus(DealStatusEnum.DOING.getCode());
                flag = withdrawSdk.signContract(user.getRealName(), user.getIdCardNo(), user.getPhone(), contractNo.toString());
            }
            if (flag) {
                sysUserContractRecordService.save(contractRecord);
            }
        }
        return HttpResponseResult.succeed(true);
    }

}

